﻿/*
 *Copyright(C) 2023 by  GYYX All rights reserved.
 *Unity版本：2021.3.10f1c1 
 *作者:Chief  
 *创建日期: 2022-09-22 
 *模块说明：
 *版本: 0.1
*/

using System;
using UnityEngine;

namespace Oroxylum
{
    /// <summary>
    /// 根据版本更新的方法；
    /// 需要保证里面的值，一帧至多计算一次，每次获取都能保证拿到最新的值；
    /// </summary>
    public class VersionValue<T> where T : struct
    {

        private T mValue;

        private int mVersion;

        public static Func<int> GetVersionMethod;

        /// <summary>
        /// 获取当前版本的方法；
        /// </summary>
        public static int CurrentVersion
        {
            get
            {
                if (GetVersionMethod != null)
                    return GetVersionMethod();
                return Time.frameCount;
            }
        }

        /// <summary>
        /// 获取当前值；
        /// </summary>
        public T Value
        {
            get
            {
                int curVersion = CurrentVersion;
                if (mVersion != curVersion)
                {
                    mValue = GetValueMethod();
                    mVersion = curVersion;
                }
                return mValue;
            }
        }

        public Func<T> GetValueMethod;

        public VersionValue(Func<int> versionMethos, Func<T> valueMethod)
        {
            GetValueMethod = valueMethod;
            GetVersionMethod = versionMethos;
        }

        public void ResetVersion() { mVersion = -1; }

    }
}