using Unity.Mathematics;
using UnityEngine;

namespace Oroxylum
{
    public static class MathExtensions
    {
        /// <summary>
        /// 平方
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static float Square(this float value) { return value * value; }

        public static float2 XZ_float2(this Vector3 v) { return new float2(v.x, v.z); }

        public static float2 XY_float2(this Vector3 pos) { return new float2(pos.x, pos.y); }

        public static float2 ZY_float2(this Vector3 pos) { return new float2(pos.z, pos.y); }

        /// <summary>
        /// 是否是有效的 Vector3；
        /// </summary>
        public static bool IsVaild(this float3 point)
        {
            if (float.IsNaN(point.x) || float.IsNaN(point.y) || float.IsNaN(point.z))
                return false;

            if (float.IsInfinity(point.x) || float.IsInfinity(point.y) || float.IsInfinity(point.z))
                return false;

            return true;
        }

        public static float3 Div(this float3 a, float3 b)
        {            
            float x = a.x / b.x;
            float y = a.y / b.y;
            float z = a.z / b.z;
            return new float3(x, y, z);
        }

        public static float2 Div(this float2 a, float2 b)
        {
            float x = a.x / b.x;
            float y = a.y / b.y;
            return new float2(x, y);
        }

        public static float3 Mul(this float3 a, float3 b)
        {
            float x = a.x * b.x;
            float y = a.y * b.y;
            float z = a.z * b.z;
            return new float3(x, y, z);
        }

        public static float2 Mul(this float2 a, float2 b)
        {
            float x = a.x * b.x;
            float y = a.y * b.y;
            return new float2(x, y);
        }


        public static long Int2ToLong(int2 v)
        {
            int q = v.x, r = v.y;
            long rt = (long)q << 32;
            rt = rt | (r & 0xffffffff);
            return rt;
        }

        public static int2 LongToInt2(long id)
        {
            int x = (int)(id >> 32);
            int y = (int)id;
            return new int2(x, y);
        }

        /// <summary>
        /// 将 float4x4 转换为 Matrix4x4；
        /// </summary>
        /// <param name="value">float4x4</param>
        /// <returns>Matrix4x4</returns>
        public static Matrix4x4 ToMatrix4X4(this float4x4 value) { return new Matrix4x4(value.c0, value.c1, value.c2, value.c3); }

        /// <summary>
        /// 将 Matrix4x4 转换为 float4x4
        /// </summary>
        /// <param name="value">Matrix4x4</param>
        /// <returns>float4x4</returns>
        public static float4x4 ToFloat4X4(this Matrix4x4 value) { return new float4x4(value[0], value[1], value[2], value[3]); }

        public static float2 XZ_float(this Vector3 p) { return new float2(p.x, p.z); }

        public static Vector3 ToVec3ByXZ(this float2 xz, float y = 0) { return new Vector3(xz.x, y, xz.y); }

        public static float2 Vec2ToFloat2(this Vector2 p) { return new float2(p.x, p.y); }
    }
}
