﻿/*
 *Copyright(C) 2020 by  Cyf All rights reserved.
 *Unity版本：2021.3.11f1c2 
 *作者:程一峰  
 *创建日期: 2022-11-10 
 *模块说明：木蝴蝶数学库：计算结果
 *版本: 1.0
*/


using Unity.Mathematics;

namespace Oroxylum
{
    /// <summary>
    /// 相交计算结果：3d直线与3dBox
    /// </summary>
    public struct IntrLine3dBox3d
    {
        public int Quantity;
        public E_IntersectionResult Result;
        public E_IntersectionType Type;
        public float LineParam0, LineParam1;
        public float3 Point0;
        public float3 Point1;

        public IntrLine3dBox3d(Line3d line, Box3d box)
        {
            Result = E_IntersectionResult.NotComputed;
            Type = E_IntersectionType.Empty;
            LineParam0 = -float.MaxValue;
            LineParam1 = float.MaxValue;
            Point0 = 0;
            Point1 = 1;
            Quantity = 0;

            DoClipping(ref LineParam0, ref LineParam1, line.Origin, line.Direction, box,
                      true, ref Quantity, ref Point0, ref Point1, ref Type);

            Result = (Type != E_IntersectionType.Empty) ?
                E_IntersectionResult.Intersects : E_IntersectionResult.NoIntersection;
        }

        public static bool DoClipping(ref float t0, ref float t1,
                         float3 origin, float3 direction,
                         Box3d box, bool solid, ref int quantity,
                         ref float3 point0, ref float3 point1,
                         ref E_IntersectionType intrType)
        {
            // Convert linear component to box coordinates.
            float3 diff = origin - box.Center;
            float3 BOrigin = new float3(
                diff.Dot(box.AxisX),
                diff.Dot(box.AxisY),
                diff.Dot(box.AxisZ)
            );
            float3 BDirection = new float3(
                direction.Dot(box.AxisX),
                direction.Dot(box.AxisY),
                direction.Dot(box.AxisZ)
            );

            double saveT0 = t0, saveT1 = t1;
            bool notAllClipped =
                Clip(+BDirection.x, -BOrigin.x - box.Extent.x, ref t0, ref t1) &&
                Clip(-BDirection.x, +BOrigin.x - box.Extent.x, ref t0, ref t1) &&
                Clip(+BDirection.y, -BOrigin.y - box.Extent.y, ref t0, ref t1) &&
                Clip(-BDirection.y, +BOrigin.y - box.Extent.y, ref t0, ref t1) &&
                Clip(+BDirection.z, -BOrigin.z - box.Extent.z, ref t0, ref t1) &&
                Clip(-BDirection.z, +BOrigin.z - box.Extent.z, ref t0, ref t1);

            if (notAllClipped && (solid || t0 != saveT0 || t1 != saveT1))
            {
                if (t1 > t0)
                {
                    intrType = E_IntersectionType.LineSegment;
                    quantity = 2;
                    point0 = origin + t0 * direction;
                    point1 = origin + t1 * direction;
                }
                else
                {
                    intrType = E_IntersectionType.Point;
                    quantity = 1;
                    point0 = origin + t0 * direction;
                }
            }
            else
            {
                quantity = 0;
                intrType = E_IntersectionType.Empty;
            }

            return intrType != E_IntersectionType.Empty;
        }

        private static bool Clip(float denom, float numer, ref float t0, ref float t1)
        {
            // Return value is 'true' if line segment intersects the current test
            // plane.  Otherwise 'false' is returned in which case the line segment
            // is entirely clipped.

            if (denom > 0)
            {
                if (numer - denom * t1 > OMath.ZeroTolerance)
                {
                    return false;
                }
                if (numer > denom * t0)
                {
                    t0 = numer / denom;
                }
                return true;
            }
            else if (denom < 0)
            {
                if (numer - denom * t0 > OMath.ZeroTolerance)
                {
                    return false;
                }
                if (numer > denom * t1)
                {
                    t1 = numer / denom;
                }
                return true;
            }
            else
            {
                return numer <= 0;
            }
        }

    }
}