﻿/*
 *Copyright(C) 2023 by GYYX All rights reserved.
 *Unity版本：2020.3.48f1 
 *作者:程一峰  
 *创建日期: 2023-06.16 
 *模块说明：多线程通用库
 *版本: 1.0
*/

using System.Collections.Concurrent;
using System.Linq;

namespace Oroxylum
{
    /// <summary>
    /// 带并行的对象池的物体；
    /// </summary>
    public class ConcurrentPool<T> where T : class, IConcurrentPoolItem, new()
    {

        protected ConcurrentBag<T> m_Bag = new ConcurrentBag<T>();

        public virtual T Get()
        {
            T obj;

            //对象池空了，创建一个；
            if (!m_Bag.TryTake(out obj))
            {
                obj = new T();
                obj.OnCreate();
            }

            if (obj == null)
                return Get();

            obj.OnGet();
            return obj;
        }

        /// <summary>
        /// 放回对象池
        /// </summary>
        /// <param name="obj"></param>
        /// <returns>放回是否成功</returns>
        public virtual bool Put(T obj)
        {
            if (obj == null)
                return false;

            if (m_Bag.Contains(obj))
                return false;

            obj.OnPut();
            m_Bag.Add(obj);
            return true;
        }

        public virtual void Clear()
        {
            //m_Bag.Clear(); Unity 2022+ 有 Clear 方法 netstandard, Version=2.1.0.0
            m_Bag = new ConcurrentBag<T>();
        }

    }
}
